/*
 * ANTs - Fourmis informatiques
 * Copyright (C) 2022 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#define __FIELD_C

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <assert.h>

#include "field.h"

/* Concret data type */
typedef struct elem {
  int object;     /* object */
  int pheromone;  /* pheromone rate */
} elem;
/* The field */
typedef struct Field {
  elem **field;  /* field */
  int xmax;      /* width */
  int ymax;      /* height */
} _Field;


/**
 * Constructor.
 */
Field field_new(int x, int y) {
  Field f = (Field)malloc(sizeof(_Field));
  if(!f) { fprintf(stderr, "Can't allocate memory (Field)\n"); exit(0); }

  /* Init memory with 0 */
  memset(f, 0, sizeof(_Field));

  /* Allocs */
  f->xmax = x;
  f->ymax = y;
  f->field = (elem **)malloc(sizeof(elem*) * y);
  for(int i=0; i<y; i++)
    f->field[i] = (elem *)malloc(sizeof(elem) * x);

  /* Inits */
  for(int i=0; i<y; i++)
    for(int j=0; j<x; j++) {
      f->field[i][j].pheromone = 0;
      if((0 <= j && j < f->xmax/4) && ((f->ymax - f->ymax/4) <= i && i < f->ymax))  /* bottom-left */
        f->field[i][j].object = FOOD;
      else if(((f->xmax - f->xmax/4) <= j && j < f->xmax) && (0 <= i && i < f->ymax/4))  /* top-right */
        f->field[i][j].object = FOOD;
      else if(((f->xmax - f->xmax/4) <= j && j < f->xmax) && ((f->ymax - f->ymax/4) <= i && i < f->ymax))  /* bottom-right */
        f->field[i][j].object = FOOD;
      else
        f->field[i][j].object = NONE;
    }

  return f;
}

/**
 * Destructor.
 */
int field_destroy(Field f) {
  /* Assertion */
  assert(f && f->field);

  /* Free memory */
  for(int i=0; i<f->ymax; i++)
    free(f->field[i]);
  free(f->field);
  free(f);

  return 1;
}


/**
 * SizeX (begin at 0).
 */
int field_xmax(Field f) {
  /* Assertion */
  assert(f && f->field);
  
  return f->xmax;
}

/**
 * SizeY (begin at 0).
 */
int field_ymax(Field f) {
  /* Assertion */
  assert(f && f->field);

  return f->ymax;
}


/**
 * Field is empty?
 */
int field_is_empty(Field f) {
  /* Assertion */
  assert(f && f->field);

  for(int i=0; i<f->ymax; i++)
    for(int j=0; j<f->xmax; j++)
      if(f->field[i][j].object)
        return 0;
  return 1;
}

/**
 * Object encounterred?
 * Or, pos (x,y) is busy?
 */
int field_is_busy(Field f, int x, int y) {
  /* Assertion */
  assert(f && f->field);
  assert(x >= 0); assert(x < f->xmax);
  assert(y >= 0); assert(y < f->ymax);

  return f->field[y][x].object;
}

/**
 * Update field (object) in pos (x, y).
 */
int field_put_down(Field f, int x, int y, int o) {
  /* Assertion */
  assert(f && f->field);
  assert(x >= 0); assert(x < f->xmax);
  assert(y >= 0); assert(y < f->ymax);

  return (f->field[y][x].object = o);
}


/**
 * Return pheromone rate on pos (x,y).
 */
double field_taux_pheromone(Field f, int x, int y) {
  /* Assertion */
  assert(f && f->field);
  assert(x >= 0); assert(x < f->xmax);
  assert(y >= 0); assert(y < f->ymax);

  return (double)f->field[y][x].pheromone/(double)MAX_PHEROMONE;
}

/**
 * Put pheromones at pos (x, y).
 */
int field_incr_pheromone(Field f, int x, int y) {
  /* Assertion */
  assert(f && f->field);

  if(f->field[y][x].pheromone < MAX_PHEROMONE)
    f->field[y][x].pheromone += 1;

  return f->field[y][x].pheromone;
}

/**
 * Pheromones are volatilizable...
 */
int field_pheromone_evaporation(Field f) {
  /* Assertion */
  assert(f && f->field);

  for(int i=0; i<f->ymax; i++)
    for(int j=0; j<f->xmax; j++)
      if(f->field[i][j].pheromone > 0)
        f->field[i][j].pheromone -= 1;
  return 1;
}

