/*
 * ANTs - Fourmis informatiques
 * Copyright (C) 2000 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

/**
 * Definition of the Ant struct., and public interface.
 */

#ifndef _ANT_H
#define _ANT_H

#include "nest.h"
#include "field.h"

/* Compass card */
#define NORTH 0
#define NORTH_EAST 1
#define EAST 2
#define SOUTH_EAST 3
#define SOUTH 4
#define SOUTH_WEST 5
#define WEST 6
#define NORTH_WEST 7

/* Orientation */
#define FORWARD 0
#define FORWARD_RIGHT 1
#define FORWARD_LEFT 2
#define RIGHT 3
#define LEFT 4
#define BACKWARD_RIGHT 5
#define BACKWARD_LEFT 6
#define BACKWARD 7

/* Follow pheromone? */
#define PHEROMONE 1
/* And/or directive research? */
#define DIRECTIVITY 1

/* An ant */
typedef struct Ant *Ant; 


/**
 * Constructor.
 * @return a new Ant.
 */
Ant ant_new(void);

/**
 * Destructor.
 * @param a  The Ant.
 * @return 1.
 */
int ant_kill(Ant a);


/**
 * Current pos X.
 * @param a  The Ant.
 * @return the pos X.
 */
int ant_posx(Ant a) ;

/**
 * Current pos Y.
 * @param a  The Ant.
 * @return the pos Y.
 */
int ant_posy(Ant a) ;

/**
 * Move on X axis.
 * @param a  The Ant to update.
 * @param n  The depl on X-axis.
 * @return the new pos X.
 */
int ant_movex(Ant a, int n);

/**
 * Move on Y axis.
 * @param a  The Ant to update.
 * @param n  The depl on Y-axis.
 * @return the new pos Y.
 */
int ant_movey(Ant a, int n);


/**
 * Current direction followed.
 * @param a  The Ant.
 * @return the direction.
 */
int ant_direction(Ant a);

/**
 * Update the direction followed.
 * @param a  The Ant to update.
 * @param d  The current direction followed.
 * @return the new direction.
 */
int ant_updt_direction(Ant a, int d);


/**
 * Am I carrying an object?
 * @param a  The Ant.
 * @return the object loaded or NONE.
 */
int ant_is_loaded(Ant a);

/**
 * Load an objet if not already loaded.
 * @param a  The Ant.
 * @return the object loaded.
 */
int ant_load(Ant a, int o);

/** 
 * Unload freight.
 * @param a  The Ant.
 * @return the object unloaded.
 */
int ant_unload(Ant a);

/**
 * Can I load?
 * @param a  The Ant.
 * @param n  The Nest.
 * @return 1 if object loaded, else 0.
 */
short ant_can_load(Ant a, Nest n);
short ant_can_load2(Ant a, Nest n);

/**
 * Can I unload?
 * @param a  The Ant.
 * @param n  The Nest.
 * @return 1 if object unloaded, else 0.
 */
short ant_can_unload(Ant a, Nest n);
short ant_can_unload2(Ant a, Nest n);


/**
 * Save objects encounterred in memory.
 * @param a  The Ant to update.
 * @param o  The current object encounterred.
 * @return the last object encounterred.
 */
int ant_add_memory(Ant a, int o);

/**
 * Compute the occupation rate of current loaded object in memory.
 * @param a  The Ant.
 * @return the occupation rate.
 */
float ant_occupation_rate(Ant a);

/**
 * Compute the occupation rate according to neighbourhood.
 * @param a  The Ant.
 * @param n  The Nest.
 * @return the occupation rate.
 */
float ant_occupation_rate2(Ant a, Nest n);


/**
 * Move the ant on its nest, then load/unload objects.
 * @param a  The Ant.
 * @param n  The Nest.
 * @return 1.
 */
int ant_move(Ant a, Nest n, int d);


/**
 * Put pheromone on the track followed by the ant.
 * @param a  The Ant.
 * @param f  The Field.
 * @return 1.
 */
int ant_put_down_pheromone(Ant a, Field f);

/**
 * Move the ant according to pheromones, then load/unload objects.
 * @param a  The Ant.
 * @param f  The Field.
 * @return 1.
 */
int ant_move_following_pheromone(Ant a, Field f);


#endif /* _ANT_H */
