/*
 * ANTs - Fourmis informatiques
 * Copyright (C) 2000 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>

#include <time.h>

#ifdef UNISTD
#include <unistd.h>
#endif
#include <sys/utsname.h>
#include <getopt.h> /* part of GNU C library */

#include "field.h"
#include "ant.h"

/* Display title */
#define field_title(xmax, ymax, nbants, step, t) do {           \
    printf("\033[2J");  /* clear screen */                      \
    printf("\033[%d;%dH", 1, 1);                                \
    printf("\nFIELD : (%dx%d) %d ants, (%d step, %d seconds)",  \
           xmax, ymax, nbants, step, t);                        \
    if(PHEROMONE) printf(" + pheromone (max %d / %d steps)",    \
                         MAX_PHEROMONE, PHEROMONE_EVAPORATION); \
    if(DIRECTIVITY) printf(" + directivity");                   \
    else printf(" + random");                                   \
    printf("\n");                                               \
    for(int j=0; j<xmax; j++) printf("=="); printf("\n");       \
  } while(0)


/**
 * Print help/usage.
 */
static int print_usage(void) {
  printf("\nUsage: findants [-seed n] [--help] [--version]\n"
         "Harvesting food by computer ants.\n\n"
         "  Options:\n"
         "  --seed n       seed for a new sequence of pseudo-random integers\n"
         "  --fieldw n     field width\n"
         "  --fieldh n     field height\n"
         "  --ants n       ants number\n"
         "  --version      output version information and exit\n"
         "  --help         display this message and exit\n"
         "\nReport bugs to klnavarro@onmail.com\n");
  return 0;
}

/**
 * Search the best track by ants.
 */
int main(int argc, char **argv) {
  int c, opt_ind=0;
  static struct option long_options[] = {
    {"seed", 1, 0, 's'},
    {"fieldw", 1, 0, 'x'},
    {"fieldh", 1, 0, 'y'},
    {"ants", 1, 0, 'a'},
    {"help", 0, 0, 'h'},
    {"version", 0, 0, 'V'},
    {0, 0, 0, 0}
  };

  /* Command line options */
  int nbants=20;
  int xmax=80, ymax=40;
  while((c=getopt_long(argc, argv, "Vhs:x:y:a:", long_options, &opt_ind))!=EOF) {
    switch(c) {
    case 's':
      srand(atoi(optarg));
      break;
    case 'x':
      xmax = atoi(optarg);
      break;
    case 'y':
      ymax = atoi(optarg);
      break;
    case 'a':
      nbants = atoi(optarg);
      break;
    case 'h':
      print_usage(); return 0;
    case 'V':
      printf("%s %s\n", argv[0], "0.2.5"); return 0;
    default:
      break;
    }
  }
  /* Others args. */
  if(optind < argc) {
  }
  else {
    /* Too few args */
  }

  /* Init field */
  Field f = field_new(xmax, ymax);
  field_title(xmax, ymax, nbants, 0, 0); field_display(f);
  
  /* Lay ants */
  Ant *a = (Ant*)malloc(sizeof(Ant) * nbants);
  for(int i=0; i<nbants; i++)
    a[i] = ant_new();

  /* Ants at work */
  time_t t0 = time(NULL);
  int step = 1;
  do {
    for(int j=0; j<nbants; j++)
      ant_move_following_pheromone(a[j], f);
    step++;
    if(step%PHEROMONE_EVAPORATION == 0) field_pheromone_evaporation(f);
    if(step%1000 == 0) {
      field_title(xmax, ymax, nbants, step, (int)difftime(time(NULL), t0));
      field_display(f);
#ifdef UNISTD
      //sleep(1);
      usleep(50000);  /* microseconds */
#endif
      if(field_is_empty(f)) break;
    }
  } while(1);

  /* Free memory */
  for(int i=0; i<nbants; i++)
    ant_kill(a[i]);
  free(a);
  field_destroy(f);
  return 0;
}




