/*
 * ANTs - Fourmis informatiques.
 * Copyright (C) 2000 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>

#include <time.h>

#ifdef UNISTD
#include <unistd.h>
#endif
#include <sys/utsname.h>
#include <getopt.h>  /* part of GNU C library */

#include "nest.h"
#include "ant.h"

/**
 * Print help/usage.
 */
static int print_usage(void) {
  printf("\nUsage: triants [-seed n] [--help] [--version]\n"
         "Sorting of the elements of a nest by combutir ants.\n\n"
         "  Options:\n"
         "  --seed n       seed for a new sequence of pseudo-random integers\n"
         "  --nestw n      nest width\n"
         "  --nesth n      nest height\n"
         "  --ants n       ants number\n"
         "  --version      output version information and exit\n"
         "  --help         display this message and exit\n"
         "\nReport bugs to klnavarro@onmail.com\n");
  return 0;
}

/**
 * Sort.
 */
int main(int argc, char **argv) {
  int c, opt_ind=0;
  static struct option long_options[] = {
    {"seed", 1, 0, 's'},
    {"nestw", 1, 0, 'x'},
    {"nesth", 1, 0, 'y'},
    {"ants", 1, 0, 'a'},
    {"help", 0, 0, 'h'},
    {"version", 0, 0, 'V'},
    {0, 0, 0, 0}
  };

  /* Command line options */
  int nbants=20;
  int xmax=80, ymax=20;
  while((c=getopt_long(argc, argv, "Vhs:x:y:a:", long_options, &opt_ind))!=EOF) {
    switch(c) {
    case 's':
      srand(atoi(optarg));
      break;
    case 'x':
      xmax = atoi(optarg);
      break;
    case 'y':
      ymax = atoi(optarg);
      break;
    case 'a':
      nbants = atoi(optarg);
      break;
    case 'h':
      print_usage(); return 0;
    case 'V':
      printf("%s %s\n", argv[0], "0.2.4"); return 0;
    default:
      break;
    }
  }
  /* Others args. */
  if(optind < argc) {
  }
  else {
    /* Too few args. */
  }

  /* Init nest */
  Nest n = nest_new(xmax, ymax);
  printf("NEST : (%dx%d) %d ants, (0 step, 0 seconds)\n", xmax, ymax, nbants); 
  for(int j=0; j<xmax; j++) printf("="); printf("\n");
  nest_display(n);
  
  /* Lay ants */
  Ant *a = (Ant*)malloc(sizeof(Ant) * nbants);
  for(int i=0; i<nbants; i++)
    a[i] = ant_new();

  /* Ants at work */
  time_t t0 = time(NULL);
  int step = 1;
  while(1) {
    for(int j=0; j<nbants; j++)
      ant_move(a[j], n, (int)(((float)rand()/(float)RAND_MAX) * 8.0));
    step++;
    if(step%1000 == 0) {
      printf("\033[2J");  /* clear screen */
      printf("\033[%d;%dH", 1, 1);
      printf("\nNEST : (%dx%d) %d ants, (%d step, %d seconds)\n", 
             xmax, ymax, nbants, step, (int)difftime(time(NULL), t0)); 
      for(int j=0; j<xmax; j++) printf("="); printf("\n");
      nest_display(n);
#ifdef UNISTD
      //sleep(1);
      usleep(50000);  /* microseconds */
#endif
    }
  }

  /* Free memory */
  for(int i=0; i<nbants; i++)
    ant_kill(a[i]);
  free(a);
  nest_destroy(n);
  return 0;
}
